#!/bin/sh
#A prxima linha reinicia usando o wish \
    exec wish "$0" "$@"


puts "Mdulo masterflex-0.3.tcl carregado!"

#****************************************************************************
#Array que vai armazenar os diferentes modelos de tubos usados com as
#respectivas razes vazo/rpm permitindo calcular os volumes bombeados
#para os diferentes tubos em diferentes rotaes
#****************************************************************************
set cte_tubo(LS_13) 0.0600
set cte_tubo(LS_14) 0.2166
set cte_tubo(LS_16) 0.7500
set cte_tubo(Ismatec_Azul_Verde) 0.1377


#############################################################################
#Criao das fontes 
#############################################################################

global fonte_grande fonte_media 

set fonte_grande {Times 24}

set fonte_media {Times 18}


#############################################################################
#Varivel global semaforo_ctrl usada para evitar conflitos no controle
#dos dois rels 
#############################################################################

set semaforo_ctrl livre


#Fonte: http://www.souzamonteiro.com/curso_tcl/aula31.html
#Procedimento para abertura da porta serial
#status_serial:
#     SERIAL_ABERTA               Porta serial aberta
#     SERIAL_ERRO_OPEN        Nao foi possivel abrir a porta serial.
#     SERIAL_ERRO_CLOSE       Nao foi possivel fechar a porta serial.
#     SERIAL_ERRO_READ        Nao foi possivel ler a porta serial.
#     SERIAL_ERRO_WRITE       Nao foi possivel escrever na porta serial.
#     SERIAL_ERRO_TIMEOUT     Ocorreu um Time Out ao tentar ler a porta serial.
#     
#O catlogo da bomba Masterflex L/S 7550 indica
# baud 4800, paridade mpar (o), 7 bits de dados e 1 bit de parada
#******************************************************************************

proc abrir_porta { nome numero } {
    
    global id_porta status_serial num_porta_ant
    
    set baud 4800
    set paridade o
    set bit_dados 7
    set bit_parada 1
    set espera 500
    
    
    set fechado [catch {tell $id_porta}]
    
    
    #Se for a primeira execuo da rotina, a varivel num_porta_ant ainda
    #no contm nenhum valor e por isso atribumos o contedo da varivel
    #$numero para evitar erro no teste if logo em seguida
    
    if { ![info exists num_porta_ant ] } { set num_porta_ant $numero }
    
    if {($fechado) || ($numero != $num_porta_ant)} {
	
	catch {close $id_porta}
	
	if {[string equal $nome "/dev/ttyS"]} {
	    set nome [append nome $numero]
	} else {
	    set nome [append nome $numero ":"]
	}
	
	set resultado_1 [ catch { set id_porta [open $nome r+] } ]
	
	set resultado_2 [catch {fconfigure $id_porta -translation \
				    binary -timeout $espera }] 
	set resultado_3 [catch {fconfigure $id_porta -mode "$baud,$paridade,$bit_dados,$bit_parada" }]
	
	puts "resultado_1: $resultado_1 e resultado_2: $resultado_2 e resultado_3: $resultado_3"
	
	if {($resultado_1 != 0) || ($resultado_2 != 0) || ($resultado_3 != 0) } {
	    set status_serial "SERIAL_ERRO_OPEN"
	    return $status_serial
	} else {
	    set status_serial "SERIAL_ABERTA"
	    set num_porta_ant $numero
	    return $id_porta
	}
    } else {
	return $id_porta
    }
}



proc bombear { porta tubo sentido vazao volume } {
    
    global cte_tubo status_op
    
    puts "Dentro do procedimento bombear"
    
    #Definindo o nmero "01" para identificar a bomba
    
    numerar_bomba $porta 01
    
    #Primeiramente zera o contador interno da bomba para podermos
    #acompanhar o volume bombeado
    
    #Comando para zerar o contador interno da bomba das rotaes realizadas
    puts -nonewline $porta "\x02P01Z0\x0d"
    flush $porta
    
    #Comando para zerar o contador interno da bomba das rotaes pendentes
    puts -nonewline $porta "\x02P01Z\x0d"
    flush $porta
 


    #Clculo da velocidade de rotao em RPM a partir da vazo (ml/min)
    #Para atender a especificao do equipamento, o resultado  formatado
    #com uma casa decimal antes de ser enviado pela porta serial

    set rpm [format "%.1f" [expr $vazao / $cte_tubo($tubo)]]

if {$rpm < 10.0} {
	tk_messageBox -message "Desculpe, mas esta vazo com o tubo $tubo \
est abaixo do limite operacional desta bomba!" 
	    return
	}

    #Clculo do nmero de rotaes para bombear o volume especificado
    #pelo usurio rotacoes=volume/k 

    set rotacoes [format "%.2f" [expr $volume / $cte_tubo($tubo)]]


if { $rotacoes >= 99999.99 } {
	tk_messageBox -message "Desculpe, mas este volume com o tubo $tubo \
 est acima do limite operacional desta bomba!" 
	    return
	}


#Enviando o comando para configurar o sentido e a vazo em rpm
#De acordo com o manual da bomba, os comandos devem ser delimitados
#pelos caracteres <STX> "\x02" e <CR> "\x0d"

    
    puts -nonewline $porta "\x02P01S$sentido$rpm\x0d"
    flush $porta

#Enviando o comando para configurar o nmero de rotaes da bomba
    puts -nonewline $porta "\x02P01V$rotacoes\x0d"
    flush $porta

    #Comando para iniciar o bombeamente conforme os parmetros
    #definidos anteriormente

    puts -nonewline $porta "\x02P01G\x0d"
    flush $porta

puts "rpm: $rpm \"P01S $sentido $rpm\" e rotaes: $rotacoes"

}



#******************************************************************************
#Procedimento para a atribuio de um nmero de identificao para a bomba
#******************************************************************************

proc numerar_bomba { porta num } {
    
    global status_serial
    
    puts -nonewline $porta "\x05"
    flush $porta
    
    set resposta [ler_resposta $porta]
    

    if {!($status_serial == "SERIAL_ERRO_TIMEOUT")} {
	
	#Verifica se na string de resposta existe a seqncia P?
	#que  enviada pela bomba enquanto ela no foi ainda numerada
	
	set numerar [regexp {P\?} $resposta texto]
	
	if {$numerar} {
	    puts -nonewline $porta "\x02P$num\x0d"
	    flush $porta
	}
    }
}

#******************************************************************************
#Procedimento para a leitura da resposta da bomba
#Segundo o manual da bomba as respostas podem ser de 3 tipos
#bsicos: strings delimitadas por <STX>xxxx<CR>, <ACK> e <NAK>
#ACK significa que o comando foi recebido sem problemas
#NAK significa que houve problemas para a execuo do comando
#Entretanto, na prtica, no foi o que observamos.
#Aparentemente, a bomba no enviou os caracterees <ACK> ou <NAK>
#******************************************************************************

proc ler_resposta { porta } {
    
    global status_serial
   
    while 1 {

        set leitura [catch {set caractere [read $porta 1]}]

	#Verifica se no houve erro de leitura
	if {$leitura} {
	    set status_serial "SERIAL_ERRO_READ"
	    return $status_serial
	}
	
        # Se nenhum byte for recebido, ocorreu um TIMEOUT.
        if {$caractere == ""} {
	    set status_serial "SERIAL_ERRO_TIMEOUT"
	    return $status_serial
        }
        
	#Continua a receber os bytes ate encontrar um caractere CR.
        if {$caractere == "\x0d"} {
	    return $palavra
        }

	# Armazena o caractere recebido.
        append palavra $caractere
    }
    
}

#******************************************************************************
#Procedimento para obter informaes sobre as condies de operao
#e retornar as informaes como uma lista
#Expresso regular que "extrai" da resposta da bomba a ltima seqncia 
#de 5 dgitos
#contendo vrias informaes sobre a bomba
#<STX>PnnIxxxxx<CR>
#         |||||Comunicao (0,1,2,3,4 ou 5)
#         ||||Bomba 
#         |||Entrada auxiliar (0=aberto, 1=fechado)
#         ||Sada Auxiliar (0=off, 1=on) 
#         |Operao (1=remota, 0=auxiliar)
#
#Siginificado dos cdigos de Comunicao: 0=sem erro, 1=erro de paridade, 
#2=erro de framing, 3=estouro de buffer, 4=comando invlido e 5=dado invlido.
#******************************************************************************

proc informar { porta tubo } {
    
    global cte_tubo
    
    #Agora envia o comando I, solicitando informaes sobre a bomba
    #e armazena a string com as informaes na varivel status
    #com o uso de uma expresso regular [0-7]+$
    
    puts -nonewline $porta "\x02P01I\x0d"
    flush $porta
    
    set resposta [ler_resposta $porta]
    
    puts "reposta: $resposta"
    
    set r1 [regexp {([0-7]+$)} $resposta status_geral]
    
    #Retira o quarto dgito da string status contendo informaes
    #sobre a bomba
    if {$r1} {
	set bomba [string index $status_geral 3]
	switch -- $bomba {
	    1 { set status_b "Bomba numerada, Aguardando comando" }
	    2 { set status_b "Aguardando comando para iniciar"    }
	    3 { set status_b "Bomba em funcionamento"             }
	    4 { set status_b "Interrupo manual"                  }
	    5 { set status_b "Sem comunicao com o motor"        }
	    6 { set status_b "Sobrecarga do motor"                }
	    7 { set status_b "Excessivo \"Feedback\" do motor"    }
	    default { set status_b "Cdigo desconhecido"          }
	}
	
	set dados [string index $status_geral 4]

	#Agora envia o comando C, solicitando informaes sobre rotaes realizadas
	#e armazena a string com as informaes na varivel status
	#com o uso de uma expresso regular [0-7]+$
	
	puts -nonewline $porta "\x02P01C\x0d"
	flush $porta
	
	set resposta [ler_resposta $porta]
	
	set r2 [regexp {([0-9]+\.[0-9]+)} $resposta rotacoes_real]
	
	if {$r2} {
	    set vol [format "%.2f" [expr $rotacoes_real * $cte_tubo($tubo)]]
	    
	    puts "rotaes: $rotacoes_real e volume: $vol"
	    
	    set retorno [list $status_b $vol $dados]
	    
	    return $retorno
	}
    }

    set retorno [list $resposta 0 1]
    return $retorno
}



proc parar { porta } {
   
    #Comando para parar a bomba
    puts -nonewline $porta "\x02P01H\x0d"
    flush $porta
   
}

#Toplevel para exibir os comandos para acionamento dos rels da bomba
#Masterflex

proc reles { } {

destroy .rele1
toplevel .rele1

destroy .rele2
toplevel .rele2


#set comando_r2 1

global status_ciclo_r1 status_ciclo_r2
global comando_r1 comando_r2
global tmp_total_r2 tmp_total_r1
global fonte_grande fonte_media 

set label_r1 "Vlvula de Descarte \n do Tq. de Aerao "
set label_r2 "Bomba de Reciclo do Lodo \n Sedimentado"

set tmp_total_r2 0
set tmp_total_r1 0


labelframe .rele1.lf -text "Rel 1" -font $fonte_media

#checkbutton .rele1.lf.r -text $label_r1 -font $fonte_media \
#    -variable comando_r1 -relief flat -command {
#    if {($bomba_em_uso == "Masterflex")} {
#	    ctrl $comando_r1 $comando_r2
#	}
#}

checkbutton .rele1.lf.r -text $label_r1 -variable comando_r1 \
    -relief flat -font $fonte_media


######################################################################

#Elementos para o controle do ciclo do rel 1
#onde o usurio define o Ciclo (ciclo), tempo que r1 permanece
#ligado (r1_lig) e o nmero de repeties de cada ciclo (repeticoes)



labelframe .rele1.lf.c -text "Durao de um Ciclo" -font $fonte_media

label .rele1.lf.c.ciclo_label -text "Ciclo (s)" -font $fonte_media

spinbox .rele1.lf.c.ciclo -from 0 -to 999 -bg white \
-textvariable ciclo_r1 -width 3 -font $fonte_media

labelframe .rele1.lf.rl -text "Tempo de acionamento do Rel 1" \
    -font $fonte_media

label .rele1.lf.rl.r_lig_label -text "Rel 2 ligado (s)" \
    -font $fonte_media

spinbox .rele1.lf.rl.r_lig -from 0 -to 999 -bg white \
-textvariable r1_lig -width 3 -font $fonte_media

labelframe .rele1.lf.rpt -text "Repeties de cada Ciclo" \
    -font $fonte_media

label .rele1.lf.rpt.l -text "Nmero de Repeties" \
    -font $fonte_media

spinbox .rele1.lf.rpt.repeticoes -from 0 -to 999999 -bg white \
    -textvariable repeticoes_r1 -width 6 -font $fonte_media

labelframe .rele1.lf.tmp_op -text "Tempo de Operao" \
    -font $fonte_media

label .rele1.lf.tmp_op.tmp_total_r -text "Tempo total (horas):" \
    -font $fonte_media

label .rele1.lf.tmp_op.tmp_total_r_n -textvar tmp_total_r1 \
    -font $fonte_media


bind .rele1.lf.rpt.repeticoes <ButtonRelease> {
    set tmp_total_r1 [calcula_tempo $repeticoes_r1 $ciclo_r1]
}

bind .rele1.lf.rpt.repeticoes <Return> {
    set tmp_total_r1 [calcula_tempo $repeticoes_r1 $ciclo_r1]
}

bind .rele1.lf.rpt.repeticoes <KP_Enter> {
    set tmp_total_r1 [calcula_tempo $repeticoes_r1 $ciclo_r1]
}

bind .rele1.lf.c.ciclo <ButtonRelease> {
    set tmp_total_r1 [calcula_tempo $repeticoes_r1 $ciclo_r1]
}

bind .rele1.lf.c.ciclo <Return> {
    set tmp_total_r1 [calcula_tempo $repeticoes_r1 $ciclo_r1]
}


bind .rele1.lf.c.ciclo <KP_Enter> {
    set tmp_total_r1 [calcula_tempo $repeticoes_r1 $ciclo_r1]
}


frame .rele1.lf.ctrl 

button .rele1.lf.ctrl.iniciar -text "Iniciar" -font $fonte_media \
    -command  {

#after 0 [list botao_iniciar_rele1 $repeticoes_r1 $r1_lig $ciclo_r1]

puts "ciclo = $ciclo_r1 r1_lig = $r1_lig repeticoes = $repeticoes_r1"

	.rele1.lf.ctrl.iniciar configure -state disable

	.rele1.lf.c.ciclo configure -state disable

	.rele1.lf.rl.r_lig configure -state disable

	.rele1.lf.rpt.repeticoes configure -state disable
	
#	after 0 [list iniciar_ciclo_r1 $repeticoes_r1 $r1_lig $ciclo_r1]

	iniciar_ciclo_r1 $repeticoes_r1 $r1_lig $ciclo_r1
        
	
#puts "\nAguardando a mudana da varivel status_ciclo_r1\n" 
	#vwait status_ciclo_r1
	
	
}



button .rele1.lf.ctrl.parar -text "Parar" -font $fonte_media \
    -command { 
	set semaforo_r1 0
	set comando_r1 0
	ctrl $comando_r1 $comando_r2
	set status_ciclo_r1 fim 
	
	.rele1.lf.ctrl.iniciar configure -state normal
	
	.rele1.lf.c.ciclo configure -state normal
	
	.rele1.lf.rl.r_lig configure -state normal
	
	.rele1.lf.rpt.repeticoes configure -state normal
	
	
}

###################################################################
#Elementos para o controle do ciclo do rel 2
#onde o usurio define o Ciclo (ciclo), tempo que r2 permanece
#ligado (r2_lig) e o nmero de repeties de cada ciclo (repeticoes)
###################################################################

labelframe .rele2.lf -text "Rel 2" -font $fonte_media

labelframe .rele2.lf.c -text "Durao de um Ciclo" -font $fonte_media

label .rele2.lf.c.ciclo_label -text "Ciclo (s)" -font $fonte_media

spinbox .rele2.lf.c.ciclo -from 0 -to 999 -bg white \
-textvariable ciclo_r2 -width 3 -font $fonte_media

labelframe .rele2.lf.rl -text "Tempo de acionamento do Rel 2" \
    -font $fonte_media

label .rele2.lf.rl.r_lig_label -text "Rel 2 ligado (s)" \
    -font $fonte_media

spinbox .rele2.lf.rl.r_lig -from 0 -to 999 -bg white \
-textvariable r2_lig -width 3 -font $fonte_media

labelframe .rele2.lf.rpt -text "Repeties de cada Ciclo" \
    -font $fonte_media

label .rele2.lf.rpt.l -text "Nmero de Repeties" \
    -font $fonte_media

spinbox .rele2.lf.rpt.repeticoes -from 0 -to 999999 -bg white \
    -textvariable repeticoes_r2 -width 6 -font $fonte_media


labelframe .rele2.lf.tmp_op -text "Tempo de Operao" \
    -font $fonte_media

label .rele2.lf.tmp_op.tmp_total_r -text "Tempo total (horas):" \
    -font $fonte_media

label .rele2.lf.tmp_op.tmp_total_r_n -textvar tmp_total_r2 \
    -font $fonte_media

bind .rele2.lf.rpt.repeticoes <ButtonRelease> {
       set tmp_total_r2 [calcula_tempo $repeticoes_r2 $ciclo_r2]
}

bind .rele2.lf.rpt.repeticoes <Return> {
set tmp_total_r2 [calcula_tempo $repeticoes_r2 $ciclo_r2]
}

bind .rele2.lf.rpt.repeticoes <KP_Enter> {
set tmp_total_r2 [calcula_tempo $repeticoes_r2 $ciclo_r2]
}

bind .rele2.lf.c.ciclo <ButtonRelease> {
set tmp_total_r2 [calcula_tempo $repeticoes_r2 $ciclo_r2]
}

bind .rele2.lf.c.ciclo <Return> {
set tmp_total_r2 [calcula_tempo $repeticoes_r2 $ciclo_r2]
}


bind .rele2.lf.c.ciclo <KP_Enter> {
set tmp_total_r2 [calcula_tempo $repeticoes_r2 $ciclo_r2]
}

frame .rele2.lf.ctrl 

button .rele2.lf.ctrl.iniciar -text "Iniciar" -font $fonte_media \
    -command  {

#after 0 [list botao_iniciar_rele2 $repeticoes_r2 $r2_lig $ciclo_r2]

puts "ciclo = $ciclo_r2 r2_lig = $r2_lig repeticoes = $repeticoes_r2"

	.rele2.lf.ctrl.iniciar configure -state disable

	.rele2.lf.c.ciclo configure -state disable

	.rele2.lf.rl.r_lig configure -state disable

	.rele2.lf.rpt.repeticoes configure -state disable
	
#	after 0 [list iniciar_ciclo_r2 $repeticoes_r2 $r2_lig $ciclo_r2]
	
	iniciar_ciclo_r2 $repeticoes_r2 $r2_lig $ciclo_r2

#	puts "\nAguardando a mudana da varivel status_ciclo_r2\n"

#	vwait status_ciclo_r2
	

}
 

button .rele2.lf.ctrl.parar -text "Parar" -font $fonte_media \
    -command { 
set semaforo_r2 0
set comando_r2 0
ctrl $comando_r1 $comando_r2
set status_ciclo_r2 fim 

.rele2.lf.ctrl.iniciar configure -state normal

.rele2.lf.c.ciclo configure -state normal

.rele2.lf.rl.r_lig configure -state normal

.rele2.lf.rpt.repeticoes configure -state normal


}

checkbutton .rele2.lf.r -text $label_r2 -variable comando_r2 \
    -relief flat -font $fonte_media


##########################################################################
#Comandos pack para exibio das telas
##########################################################################

pack .rele1.lf.r .rele2.lf.r 


pack .rele1.lf.c.ciclo_label .rele1.lf.c.ciclo -side left -expand yes -fill both


pack .rele1.lf.rl.r_lig_label .rele1.lf.rl.r_lig -side left -expand yes -fill both

pack .rele1.lf.rpt.l .rele1.lf.rpt.repeticoes -side left -expand yes -fill both

pack .rele1.lf.tmp_op.tmp_total_r .rele1.lf.tmp_op.tmp_total_r_n -side left -expand yes -fill both

pack .rele1.lf.c .rele1.lf.rl .rele1.lf.rpt  .rele1.lf.tmp_op .rele1.lf.ctrl  -expand yes -fill both  -pady 5


pack .rele1.lf.ctrl.iniciar .rele1.lf.ctrl.parar -side left -expand yes -fill both



pack .rele2.lf.c.ciclo_label .rele2.lf.c.ciclo -side left -expand yes -fill both


pack .rele2.lf.rl.r_lig_label .rele2.lf.rl.r_lig -side left -expand yes -fill both


pack .rele2.lf.rpt.l .rele2.lf.rpt.repeticoes -side left -expand yes -fill both


pack .rele2.lf.tmp_op.tmp_total_r .rele2.lf.tmp_op.tmp_total_r_n -side left -expand yes -fill both


pack .rele2.lf.c .rele2.lf.rl .rele2.lf.rpt  .rele2.lf.tmp_op .rele2.lf.ctrl   -expand yes -fill both  -pady 5


pack .rele2.lf.ctrl.iniciar .rele2.lf.ctrl.parar -side left -expand yes -fill both


pack .rele1.lf .rele2.lf  -expand yes -fill both -pady 10


#Ciclo (s)          __   
#Rel 2 ligado (s)  __
#Repeties         __
#Tempo total       
#Iniciar

}



#Procedimento para envio dos comandos para os rels da bomba Masterflex
#A varivel global id_porta havia sido criada pelo procedimento abrir_porta
#e est sendo usada aqui para verificar se a porta serial j foi aberta
#A verificao  feita com o comando "set fechado [catch {tell $id_porta}]"
#Caso a porta j esteja aberta, usa o descrito disponvel, caso contrrio
#cria novo descritor para executar o comando e fecha novamente.
#Isto permite controlar os reles a qualquer momento, e no criar novo
#descritor para a porta serial sem necessidade

proc ctrl {r1 r2} {
    
    global porta_serial num_porta_serial
    
    global id_porta
    
    set fechado [catch {tell $id_porta}]
    
    if {$fechado} {

	set porta [abrir_porta $porta_serial $num_porta_serial]
	
	puts "$r1 $r2 $porta"
	
	if {($porta != "SERIAL_ERRO_OPEN")} {
	    
	    puts -nonewline $porta "\x02P01O$r1$r2\x0d"
	    flush $porta
	    
	    close $porta
	}
    } else {
	  

	puts -nonewline $id_porta "\x02P01O$r1$r2\x0d"
	flush $id_porta  
	puts "$r1 $r2 $id_porta"
	
    }

}


############################################################################
#Procedimento para controle do ciclo de operao do rel 2
############################################################################

proc iniciar_ciclo_r2 { rep t_r2 ciclo } {
    
    global comando_r1 comando_r2
    global status_ciclo_r2
    global semaforo_r2

    set semaforo_r2 1
    
global contador_r2

    if { $t_r2 >= $ciclo } {
	    tk_messageBox -message "Desculpe, mas o tempo de ligao do rel 2 deve ser menor que o Ciclo!"
	    set status_ciclo_r2 fim
	    return
	}
	
	set t_r2_lig [expr $t_r2 * 1000]
	set t_r2_des [expr [expr $ciclo * 1000] - $t_r2_lig]
	
#	set status_ciclo_r2 inicio

	set contador_r2 0	

	ligar_r2 $t_r2_lig $t_r2_des $rep

	
}

proc ligar_r2 { t_lig t_des ciclos } {
    
    global contador_r2 status_ciclo_r2
    global comando_r1 comando_r2
    global semaforo_r2

    set comando_r2 1

    ctrl $comando_r1 $comando_r2
    

    set id_after [ after $t_lig desligar_r2 $t_lig $t_des $ciclos ]
    
        
    if { ($contador_r2 == $ciclos) || ($semaforo_r2 == 0) } {
	    after cancel $id_after
	    set status_ciclo_r2 fim
	    set comando_r2 0
	    ctrl $comando_r1 $comando_r2
	}
}

proc desligar_r2 { t_lig t_des ciclos } {

global contador_r2 status_ciclo_r2
global comando_r1 comando_r2
global semaforo_r2

set comando_r2 0

ctrl $comando_r1 $comando_r2

set id_after [ after $t_des ligar_r2 $t_lig $t_des $ciclos ] 

incr contador_r2

if { ($contador_r2 == $ciclos) || ($semaforo_r2 == 0) } {
    after cancel $id_after
    set status_ciclo_r2 fim
    .rele2.lf.ctrl.iniciar configure -state normal
    
    .rele2.lf.c.ciclo configure -state normal
    
    .rele2.lf.rl.r_lig configure -state normal
    
    .rele2.lf.rpt.repeticoes configure -state normal
}


}

############################################################################
#Procedimento para controle do ciclo de operao do rel 1
############################################################################

proc iniciar_ciclo_r1 { rep t_r1 ciclo } {
    
    global comando_r1 comando_r2
    global status_ciclo_r1
    global semaforo_r1
    global contador_r1



    if { $t_r1 >= $ciclo } {
	    tk_messageBox -message "Desculpe, mas o tempo de ligao do rel 2 deve ser menor que o Ciclo!"
	    set status_ciclo_r1 fim
	    return
	}

	set semaforo_r1 1
	
	set t_r1_lig [expr $t_r1 * 1000]
	set t_r1_des [expr [expr $ciclo * 1000] - $t_r1_lig]
	
#	set status_ciclo_r1 inicio

	set contador_r1 0	

	ligar_r1 $t_r1_lig $t_r1_des $rep

	
}

proc ligar_r1 { t_lig t_des ciclos } {
    
    global contador_r1 status_ciclo_r1
    global comando_r1 comando_r2
    global semaforo_r1

    set comando_r1 1

    ctrl $comando_r1 $comando_r2
    
    set id_after [ after $t_lig desligar_r1 $t_lig $t_des $ciclos ]
    
       
    if { ($contador_r1 == $ciclos) || ($semaforo_r1 == 0) } {
	    after cancel $id_after
	    set status_ciclo_r1 fim
	    set comando_r1 0
	    ctrl $comando_r1 $comando_r2
	}
}

proc desligar_r1 { t_lig t_des ciclos } {

global contador_r1 status_ciclo_r1
global comando_r1 comando_r2
global semaforo_r1

set comando_r1 0

ctrl $comando_r1 $comando_r2

set id_after [ after $t_des ligar_r1 $t_lig $t_des $ciclos ] 

incr contador_r1

if { ($contador_r1 == $ciclos) || ($semaforo_r1 == 0) } {
    after cancel $id_after
    set status_ciclo_r1 fim
    .rele1.lf.ctrl.iniciar configure -state normal
    
    .rele1.lf.c.ciclo configure -state normal
    
    .rele1.lf.rl.r_lig configure -state normal
    
    .rele1.lf.rpt.repeticoes configure -state normal
    
}


}





proc calcula_tempo {rep ciclo} {
return [format "%.2f" [expr $rep * $ciclo / 3600.00]]
}
    
    reles
    
    